-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
--Mandatory controller parameters
M.type = "main"
M.relevantDevice = nil
M.engineInfo = {}
M.fireEngineTemperature = 0
M.throttle = 0
M.brake = 0
M.clutchRatio = 0

--LOCAL--
local modes = {"Direct", "Gradual"}

--API--
M.throttle = 0
M.elevatorInput = 0
M.rudderInput = 0
M.aileronInput = 0
M.aileronAxis = false
M.elevatorAxis = false
M.rudderAxis = false

local elevatorMode = 1
local aileronMode = 1
local rudderMode = 0
local propPairs = {}

local engineBreakBeam = nil
local engineBroken = false

local RPMsmoother = newExponentialSmoothing(500)

M.toggleElevatorMode = function()
  elevatorMode = elevatorMode + 1
  if elevatorMode >= #modes then elevatorMode = 0 end
  
  --what is it tho?
  gui.message("New elevator mode: " .. modes[elevatorMode+1], 3, "Aircraft")
end

M.toggleAileronMode = function()
  aileronMode = aileronMode + 1
  if aileronMode >= #modes then aileronMode = 0 end
  
  --what is it tho?
  gui.message("New aileron mode: " .. modes[aileronMode+1], 3, "Aircraft")
end

M.toggleRudderMode = function()
  rudderMode = rudderMode + 1
  if rudderMode >= #modes then rudderMode = 0 end
  
  --what is it tho?
  gui.message("New rudder mode: " .. modes[rudderMode+1], 3, "Aircraft")
end
-------
local function init()
  --init propeller
  propPairs = {}
  if v.data.pistonprop ~= nil then 
    print("piston prop found!!!!!!")
    for temp,pair in pairs(v.data.pistonprop.pairs) do
      local realPair = {}
      for temp2, pr in pairs(pair) do
        for temp3, node in pairs(v.data.nodes) do
          if node.name == pr then table.insert(realPair, node) end
        end
      end
      table.insert(propPairs, realPair)
    end
  end
  
  for k,bm in pairs(v.data.beams) do
    if bm.name ~= nil and bm.name == "propshaft" then
      print("GOT BEAM")
      engineBreakBeam = bm
    end
  end
  
  --init stuff
  engineBroken = false
  
  --init electrics
  electrics.values.flaps = 0
  electrics.values.elevator = 0
  electrics.values.rudder = 0
  electrics.values.aileron = 0
  electrics.values.aircraft_throttle = 0.01
  electrics.values.altitudeFT = 0
  electrics.values.rollDeg = 0
  electrics.values.pitchDeg = 0
  electrics.values.yawDeg = 0
end

local function updateGFX(dt)
  electrics.values.throttle = math.min(math.max(input.throttle, 0), 1)
  electrics.values.brake = math.min(math.max(input.brake, 0), 1)
  electrics.values.clutch = math.min(math.max(input.clutch, 0), 1)
  electrics.values.clutchRatio = 1 - electrics.values.clutch
  electrics.values.gear = "N"
  electrics.values.gearIndex = 0
  electrics.values.rpm = 0
  electrics.values.oiltemp = 0
  electrics.values.watertemp = 0
  electrics.values.fuel =  0
  electrics.values.lowfuel = false
  electrics.values.checkengine = false
  electrics.values.ignition = true
  electrics.values.engineThrottle = 0
  electrics.values.running = electrics.values.ignition
  
  M.engineInfo = {
    0
    , 0
    , 1000
    , 1000
    , 0
    , 0
    , 0
    , 0
    , 0
    , 0
    , obj:getVelocity():length()
    , 0
    , 0
    , "manual"
    , obj:getID()
    , 0
    , 0
  } 
  
  --update some electrics
  local dirVector = obj:getDirectionVector()
  local dirVectorUp = obj:getDirectionVectorUp()
  local position = obj:getPosition()

  electrics.values.rollDeg = (dirVectorUp.x * -dirVector.y + dirVectorUp.y * dirVector.x) * 57.2958
  electrics.values.pitchDeg = dirVector.z * 57.2958
  electrics.values.yawDeg = obj:getDirection() * 57.2958
  electrics.values.altitudeFT = position.z * 3.28084
  
  
  --update control surfaces
  if not M.elevatorAxis then
    if elevatorMode == 0 then
      electrics.values.elevator = M.elevatorInput
    else
      local newValue = electrics.values.elevator + M.elevatorInput * dt
      electrics.values.elevator = math.min(math.max(newValue, -1), 1)
    end
  end
  
  if not M.rudderAxis then
    if rudderMode == 0 then
      electrics.values.rudder = M.rudderInput
    else
      local newValue = electrics.values.rudder + M.rudderInput * dt
      electrics.values.rudder = math.min(math.max(newValue, -1), 1)    
    end
  end
  
  if not M.aileronAxis then
    if aileronMode == 0 then
      electrics.values.aileron = M.aileronInput
    else
      local newValue = electrics.values.aileron + M.aileronInput * dt
      electrics.values.aileron = math.min(math.max(newValue, -1), 1)    
    end
  end
  
  --actual aircraft shit
  if M.throttle ~= 0 then
    electrics.values.aircraft_throttle = electrics.values.aircraft_throttle +  (dt * M.throttle)
    if electrics.values.aircraft_throttle > 0.99 then electrics.values.aircraft_throttle = 0.99 end  --hack for sounds
    if electrics.values.aircraft_throttle < 0 then electrics.values.aircraft_throttle  = 0 end
    
  end
  
  --engine bork 
  if engineBreakBeam ~= nil and (obj:beamIsBroken(engineBreakBeam.cid) or obj:getBeamLengthRefRatio(engineBreakBeam.cid) < 0.975) or obj:inWater(engineBreakBeam.id1) or obj:inWater(engineBreakBeam.id2) then
      engineBroken = true
  end
  
  if engineBroken == true then
      electrics.values.aircraft_throttle = 0
      M.throttle = 0
  end
  --debuuuuugz!!
  if input.keys.E then
    electrics.values.rocket1 = 1
    for k,v in pairs(v.data.beams) do
      if v.breakGroup == "rocketAttach1" then obj:breakBeam(v.cid) end
    end
  end
  
  electrics.values["aircraft_rpm"] = RPMsmoother:get(electrics.values.aircraft_throttle)
end

local function update()
  if v.data.pistonprop == nil then return end
  
  for temp,pair in pairs(propPairs) do
    local id1 = pair[1].cid
    local id2 = pair[2].cid
    
    if math.abs(obj:getNodeVelocity(id2, id1)) < v.data.pistonprop.maxVelocity then
      obj:applyForce(id2, id1 , (v.data.pistonprop.thrust * electrics.values.aircraft_throttle) * v.data.pistonprop.direction)
    end
    
  end
end

M.init = init
M.update = update
M.updateGFX = updateGFX

--Mandatory main controller API
M.shiftUp = nop
M.shiftDown = nop
M.shiftToGearIndex = nop
M.cycleGearboxModes = nop
M.setGearboxMode = nop
M.setStarter = nop
M.setFreeze = nop
M.sendTorqueData = nop
-------------------------------

return M